"""
    Created on 2012-01-27
    @author: jldupont
"""
import logging, os, signal
from time import sleep
import psutil


def run(prefix=None, polling_interval=None
        ,force_kill=None
        ,username=None
        ,ppid=None
        ,**_
        ):

    def prefix_match(entry):
        if prefix is None:
            return True
        
        bns=map(os.path.basename, entry.cmdline)
        flist=filter(lambda x:x.startswith(prefix), bns)
        return len(flist)>0
    

    def filtre(pentry):
        user_match=(username is None) or (username==pentry.username)
        name_match=(prefix is None)   or (pentry.name.startswith(prefix))
        ppid_match=(ppid is None)     or (pentry.ppid==ppid)
        pmatch=prefix_match(pentry)
        
        return ppid_match and (name_match or pmatch) and user_match


    liste=[]
    this_pid=os.getpid()
    this_ppid=os.getppid()
    logging.info("Process pid: %s" % this_pid)
    logging.info("Parent pid:  %s" % this_ppid)
    logging.info("Starting loop...")
    while True:
        if os.getppid()!=this_ppid:
            logging.warning("Parent terminated... exiting")
            break
        
        #deprecated in psutil
        #plist=psutil.get_process_list()
        #flist=filter(filtre, plist)
        
        for proc in psutil.process_iter():
            
            ## make sure we get a fresh copy
            p=psutil.Process(proc.pid)
            
            if filtre(p):                
                cmdline=p.cmdline

                ## skip ourself!                
                if p.pid != this_pid:
                    user=p.username
                    
                    details="pid '%s' '%s' : %s" % (p.pid, user, cmdline)
                    do_kill(p.pid, details, force_kill)
                    
        
        logging.debug("Sleeping...")
        sleep(polling_interval)


def do_kill(pid, details, force_kill):
    if not force_kill:
        logging.info("Would kill %s" % details)
    else:
        try:
            os.kill(pid, signal.SIGTERM)
            logging.info("Killed %s" % details)
        except:
            try:
                os.kill(pid, signal.SIGKILL)
                logging.info("Killed %s" % details)
            except:
                logging.warning("Couldn't kill %s" % details)
    



"""
    Created on 2012-01-27
    @author: jldupont
"""
import logging, sys, json, os
from time import sleep

ONE_SECOND=1

def stdout(jo):
    sys.stdout.write(json.dumps(jo)+"\n")
        

def run(topic_name=None
        ,separate_msg_marker=False
        ,suppress_second_marker=False
        ,seconds_count=None
        ,tick_at_start=False
        ,**_
        ):

    ### elapsed
    d={"topic": topic_name
       ,"sec": 0
       ,"min": 0
       ,"hour": 0
       ,"day": 0
       ,"min_marker": False
       ,"hour_marker": False
       ,"day_marker": False
    }
    

    ppid=os.getppid()        
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid:  %s" % ppid)
    logging.info("Starting loop...")
    
    if tick_at_start:
        send_tick()
        
    
    while True:
        if os.getppid()!=ppid:
            logging.warning("Parent terminated... exiting")
            break

        if seconds_count is None:
            d=mode_default(d, suppress_second_marker, separate_msg_marker)
        else:
            mode_interval(seconds_count)
            
def mode_interval(interval_in_seconds):
    
    sleep(interval_in_seconds)
    send_tick()
        

def send_tick():
    stdout({"topic":"tick"})


def mode_default(d, suppress_second_marker, separate_msg_marker):

    d["min_marker"]=False
    d["hour_marker"]=False
    d["day_marker"]=False
    
    sec=d["sec"]+1
    d["min_marker"]=(sec==60)
    d["sec"]=sec % 60
    
    if d["min_marker"]:
        _min=d["min"]+1
        d["hour_marker"]=(_min==60)
        d["min"]=_min % 60
        
        if d["hour_marker"]:            
            hour=d["hour"]+1
            d["day_marker"]=(hour==24)
            d["hour"]=hour % 24
            
            if d["day_marker"]:                    
                d["day"]=d["day"]+1
    
    #logging.debug(" sec(%s) min_marker(%s) hour_marker(%s) day_marker(%s)" % (sec, min_marker, hour_marker, day_marker))
    if suppress_second_marker:
        if not d["min_marker"] and not d["hour_marker"] and not d["day_marker"]:
            sleep(ONE_SECOND)
            return d
           
    try:
        stdout(d)
        
        if separate_msg_marker:
            if d["min_marker"]:
                stdout({"topic":"min_marker"})
            if d["hour_marker"]:
                stdout({"topic":"hour_marker"})
            if d["day_marker"]:
                stdout({"topic":"day_marker"})
    except:
        raise Exception("Exiting... probably broken pipe")
    
    sleep(ONE_SECOND)

    return d

